import * as React from 'react';
import { Classes, InputGroup, NumericInput, INumericInputProps } from '@blueprintjs/core';
import { WidgetProps } from 'react-jsonschema-form';
import { JSONSchema6 } from 'json-schema';

type Props = WidgetProps & {
  registry: any;
  rawErrors: any;
  inputProps: any;
  type: any;
  options: Partial<{
    fill: boolean;
  }>
}

export class BaseInput extends React.Component<Props> {
  render() {
    const {
      value,
      readonly,
      disabled,
      autofocus,
      onBlur,
      onFocus,
      options,
      schema,
      formContext,
      registry,
      rawErrors = [],
      ...inputProps
    } = this.props;

    const {
      fill = true
    } = options;

    Object.assign(inputProps, inputTypeProps(this.props));

    if (inputProps.type === 'number') {
      return (
        <NumericInput
          readOnly={readonly}
          autoFocus={autofocus}
          disabled={disabled}
          value={value == null ? '' : value}
          defaultValue={undefined}
          onValueChange={this._onValueChange}
          intent={rawErrors.length ? 'danger' : 'none'}
          max={schema.maximum}
          min={schema.minimum}
          {...inputProps}
          {...inputTypeProps(this.props)}
          // Prevent wrapped input element getting an additional spinner
          type={undefined}
          {...stepProps(schema)}
          {...{ fill }}
        />
      );
    }

    return (
      <InputGroup
        className={`${fill ? Classes.FILL : ''}`}
        readOnly={readonly}
        autoFocus={autofocus}
        disabled={disabled}
        value={value == null ? '' : value}
        defaultValue={undefined}
        intent={rawErrors.length ? 'danger' : 'none'}
        {...inputProps}
        {...inputTypeProps(this.props)}
        onChange={this._onChange}
        onBlur={this._onBlur}
        onFocus={this._onFocus}
      />
    )
  }

  protected _onChange: React.ChangeEventHandler<HTMLInputElement> = event =>
    this.props.onChange(event.target.value === '' ? (this.props.options as any).emptyValue : event.target.value)

  protected _onBlur: React.EventHandler<any> = event =>
    this.props.onBlur && this.props.onBlur(this.props.id, event.target.value)

  protected _onFocus: React.EventHandler<any> = event =>
    this.props.onFocus && this.props.onFocus(this.props.id, event.target.value)

  protected _onValueChange: INumericInputProps['onValueChange'] = value =>
    this.props.onChange(value)

}

const inputTypeProps = (props: WidgetProps & any): any => {
  if (props.options.inputType) {
    return { type: props.options.inputType };
  }

  if (!props.type) {
    if (props.schema.type === 'number') {
      return { type: 'number', step: 'any' };
    }

    if (props.schema.type === 'integer') {
      return { type: 'number', step: '1' };
    }

    return { type: 'text' };
  }
}

const stepProps = (schema: JSONSchema6): Partial<INumericInputProps> =>
  schema.multipleOf ? {
    stepSize: schema.multipleOf,
    majorStepSize: schema.multipleOf * 10,
    minorStepSize: null
  } : {
      minorStepSize: 0.01
    }
